#include <stdio.h>
#include <stdlib.h>
#include <ifaddrs.h>
#include <netdb.h>
#include <string.h>
#include <sys/socket.h>

#include "getaddr.h"

static uint32_t address, netmask;

void initaddr (char * iface)
{
  /* Base address of first interface */
  struct ifaddrs * ifaddr;

  /* Current interface */
  struct ifaddrs * ifa;

  /* Address socket address structure */
  struct sockaddr_in * sa;

  /* Netmask socket address structure */
  struct sockaddr_in * nm;

  address = netmask = 0;

  if (getifaddrs(&ifaddr) == -1)
    {
      perror("getifaddrs");
      exit(EXIT_FAILURE);
    }

  for (ifa = ifaddr ; ifa != NULL && address == 0 ; ifa = ifa->ifa_next)
    {
      if (ifa->ifa_addr == NULL)
	continue;

      if (ifa->ifa_addr->sa_family == AF_INET && (strncmp(ifa->ifa_name, iface, 16) == 0))
	{
	  sa = (struct sockaddr_in *) ifa->ifa_addr;
	  nm = (struct sockaddr_in *) ifa->ifa_netmask;
	  address = (uint32_t) sa->sin_addr.s_addr;
	  netmask = (uint32_t) nm->sin_addr.s_addr;
	}
    }

  freeifaddrs(ifaddr);

  if ((address && netmask) == 0)
    {
      fprintf(stderr, "[getaddr] Interface %s not found.\n", iface);
      exit(EXIT_FAILURE);
    }
}

uint32_t getAddress (char * iface)
{
  if (address == 0)
    initaddr(iface);
  return address;
}

uint32_t getNetmask (char * iface)
{
  if (netmask == 0)
    initaddr(iface);
  return netmask;
}
