#include <stdio.h>
#include <stdlib.h>
#include <netinet/ip.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <argp.h>
#include <stdbool.h>

#include "ospf.h"
#include "print.h"
#include "verbose.h"

const char * argp_program_bug_address = "wjholden@gmail.com";
const char * argp_program_version = "0.1";

int sd;
bool verbose = false;

static int parse_opt (int key, char * arg, struct argp_state * state)
{
  arg = arg;      // suppress -Werror=unused-value
  state = state;  // suppress -Werror=unused-value
  
  switch (key)
    {
    case 'v': verbose = true; break;
    }
  return 0;
}

struct argp_option argument_options[] =
  {
    { 0, 0, 0, 0, "Program Options:", 5 },
    { "verbose", 'v', 0, 0, "explain what is being done", 5},

    { 0 }
  };

int main (int argc, char ** argv)
{
  uint8_t buffer[65536];
  unsigned int saddr_size;
  int data_size;
  struct sockaddr saddr;

  struct argp argp = { argument_options, parse_opt, "",
		       "Receive an OSPF Hello Packet.\v"
		       "Send the packet in a separate terminal with the Hello program.", 0, 0, 0};
  argp_parse (&argp, argc, argv, 0, 0, 0);

  sd = socket(AF_INET, SOCK_RAW, IPPROTO_OSPF);

  if (sd < 0)
    {
      perror("socket() failed");
      exit(EXIT_FAILURE);
    }
  
  while (1)
    {
      saddr_size = sizeof(saddr);
      data_size = recvfrom(sd, buffer, 65536, 0, &saddr, &saddr_size);
      if (data_size < 0)
	{
	  perror("recvfrom");
	  fprintf(stderr, "Failed to get packets.\n");
	  return(EXIT_FAILURE);
	}
      struct iphdr * ip;
      struct OSPFheader * header;
      struct OSPFhello * hello;

      ip = (struct iphdr *) buffer;
      header = (struct OSPFheader *) (buffer + (ip->ihl * 4));
      hello = (struct OSPFhello *) (buffer + (ip->ihl * 4) + (24));

      printIP(ip);
      printOSPFHeader(header);
      printOSPFHello(hello);
    }
  close(sd);
  return (EXIT_SUCCESS);
}
